import net.jqwik.api.Arbitraries
import net.jqwik.api.Arbitrary
import net.jqwik.api.Combinators
import net.jqwik.api.ForAll
import net.jqwik.api.Property
import net.jqwik.api.Provide


class Issue206Reproducer {

    enum class FieldType { Numeric, String }

    @Property
    fun reproducer(@ForAll("pathological") map: Map<Int, String>) {
    }

    @Provide("pathological")
    fun provider(): Arbitrary<Map<Int, String>> {
        val entryCount = Arbitraries.integers().between(1, 50)
        val fieldMapArbitrary: Arbitrary<Map<Int, String>> = entryCount.flatMap { entryCount ->
            val fieldIdsList = Arbitraries.integers().between(1, 99)
                .list().ofSize(entryCount)
            val fieldTypesList = Arbitraries.defaultFor(FieldType::class.java)
                .list().ofSize(entryCount)
            val stringList = Arbitraries.strings().ofMinLength(1).alpha().numeric()
                .list().ofSize(entryCount)
            val numericList = Arbitraries.integers().greaterOrEqual(0)
                .list().ofSize(entryCount)

            Combinators
                .combine(fieldIdsList, fieldTypesList, stringList, numericList)
                .`as` { ids, types, strings, ints ->
                    val values = ids.mapIndexed { index, fieldId ->
                        if (types[index] == FieldType.Numeric) ints[index].toString()
                        else strings[index]
                    }
                    mutableMapOf<Int, String>().plus(ids.zip(values))
                }
        }

        return fieldMapArbitrary
    }
}
